import java.io.*;
import java.util.*;
import minerva.*;

/* *****************************************************************************
 *                                                                             *
 * Copyright (c) 1999 Paolo Atzeni, Alessandro Masci, Giansalvatore Mecca,     *
 * Crescenzi Valter and  Department of "Informatica ed Automazione" of         *
 * University "Roma Tre", Rome, ITALY.                                         *
 *                                                                             *
 * All Rights Reserved.                                                        *
 *                                                                             *
 * Permission to use, copy, and distribute this software and its documentation *
 * documentation for NON-COMMERCIAL purposes and without fee is hereby granted *
 * provided  that this copyright notice appears in all copies.                 *
 *                                                                             *
 * THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF  *
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR *
 * NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED   *
 * BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE   *
 * OR ITS DERIVATIVES.                                                         *
 *                                                                             *
 ******************************************************************************* */

/***********************************/
/**** Minerva Wrapper Generator ****/
/***********************************/

public class mwg extends Object
{

  public final static String MWG_JAVA_COMPILER_KEY = "minerva.mwg.compiler";
  public  static String PROPERTIES_FILE = "minerva/minerva.properties";
  private static String MWG_JAVA_COMPILER_VALUE = "javac";

  final private static void MESSAGE() {
    System.out.println();
    System.out.println("Usage: java mwg INPUTFILE OUTPUTWRAPPER");
    System.out.println("Minerva Wrapper Generator (Araneus Project).");
    System.out.print("Generate a java wrapper taking INPUTFILE");
    System.out.println(" specification in minerva formalism.");
    System.out.print("  (more info about Araneus Project at:");
    System.out.println(" http://www.dia.uniroma3.it/Araneus)");
  }

  public static void generate(String[] compiler, String[] argv) throws IOException,ParseException,Exception {
    main(argv);
  }
  
  public static void main(String[] argv) throws IOException,ParseException,Exception
  {
    if (argv.length!=2) { 
      MESSAGE();
      return;
    }
    InputStream  source = new FileInputStream(argv[0]);
    OutputStream dest;
    dest = new FileOutputStream(argv[1]);
    MinervaCompiler compiler = new MinervaCompiler(source,dest);
    compiler.compile(new File(argv[1]));

    System.out.println("Trying to compile java target code. . .");
    ByteArrayOutputStream javacOut = new ByteArrayOutputStream();
    PrintStream pout = new PrintStream(javacOut);
    int exitValue = -1;
    boolean exitValueSet= false;
  
    Properties defaults = new Properties();
    defaults.setProperty(MWG_JAVA_COMPILER_KEY,MWG_JAVA_COMPILER_VALUE);
    try {
      Properties properties = new Properties(defaults);
      String classpath = System.getProperty("java.class.path");
      StringTokenizer cpTokenizer = 
	new StringTokenizer(classpath,File.pathSeparator);
      boolean found = false;
      File propertiesFile = null;
      while (cpTokenizer.hasMoreTokens()) {
	String propertiesFileName = cpTokenizer.nextToken();
	propertiesFile = new File(propertiesFileName,PROPERTIES_FILE);
	if (propertiesFile.exists()) {
	  found = true;
	  break;
	}
      }
      if (found) {
	try {
	  System.err.println("Using "+propertiesFile+" properties file");
	  properties.load(new FileInputStream(propertiesFile));
	}
	catch (IOException ioe) {
	  System.out.println("Cannot load minerva properties file "+
			     propertiesFile+": using defaults");
	}
      }
      else System.out.println("Cannot found minerva properties file: using defaults");

      String commandline = properties.getProperty(MWG_JAVA_COMPILER_KEY);
      StringTokenizer commandTokens = new StringTokenizer(commandline);
      String[] javacArgs = new String[commandTokens.countTokens()+1];
      for(int i=0; i<javacArgs.length-1; i++)
	javacArgs[i] = commandTokens.nextToken();
      javacArgs[javacArgs.length-1] = argv[1];
      /*
      System.err.print("Command line used to compiler target java code: ");
      for(int i=0; i<javacArgs.length; i++)
	System.err.print(javacArgs[i]+" ");
      System.err.println();
      */
      Process p = Runtime.getRuntime().exec(javacArgs);
      BufferedReader 
	in = new BufferedReader(new InputStreamReader(p.getInputStream())),
	err = new BufferedReader(new InputStreamReader(p.getErrorStream()));
      String line = null;

      while ((line = err.readLine()) != null) {
	pout.println(line);
      }
	  
      try {
	p.waitFor();
	exitValue = p.exitValue();
	exitValueSet = true;
      }
      catch (InterruptedException ix) {
      }
    }
    catch (IOException iox) {
      iox.printStackTrace(pout);
    }
    
    if (!exitValueSet || exitValue != 0) {
      throw new Exception("Compilation error:\n"+javacOut.toString());
    }

    source.close();
    dest.close();
    return;
  }

} /* End of Class mwg */



