/* ***********************************************************************************
 *                                                                                   *
 * Copyright (c) 1997, 1998, 1999                                                    *
 * Araneus Group and Department of "Informatica ed Automazione"                      *
 * of Universita' "Roma Tre", Rome, ITALY.                                           *
 *                                                                                   *
 * All Rights Reserved.                                                              *
 *                                                                                   *
 * Permission to use, copy, and distribute this software and its documentation for   *
 * NON-COMMERCIAL purposes and without fee is hereby granted provided  that this     *
 * copyright notice appears in all copies.                                           *
 *                                                                                   *
 * THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE    *
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED     *
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR               *
 * NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY      *
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR  ITS    *
 * DERIVATIVES.                                                                      *
 *                                                                                   *
 *********************************************************************************** */


package shell;

import java.io.*;
import java.lang.*;
import java.lang.reflect.*;
import java.util.*;
import mwg;

/*
 *    class MinervaShell
 *
 *    "command shell for Minerva Wrapper Generator"
 * 
 *    (v. 1.0) V. Crescenzi
 *   
 */

class MinervaShell {

  private static String[] compiler = null;
  private static File NFfile = null;
  private static String root = null;
  private static File destFile = null;
  private static List sources = new LinkedList();
  private static LineNumberReader in = new LineNumberReader(new InputStreamReader(System.in));
  private static boolean compiled = false;
  private static boolean something_to_save = false;

  private static String NFFILE   = "NF:";
  private static String ROOT     = "ROOT:";
  private static String DEST     = "DEST:";
  private static String COMPILER = "COMPILER:";
  private static String SOURCES  = "SOURCES:";
  
  public static void main(String argv[]) throws IOException {

    if (argv.length==0) {
      System.out.println("You can specify a file containing shell "+
			 "configuration information in this form:");
      configMessage();
    } 
    else if (argv.length==1) {
      parseConfigFile(new File(argv[0]));
    } 
    else {
      System.out.println("Wrong number of arguments.");
      System.out.println("Usage: java shell.MinervaShell [<configFile>]");
      System.out.println("  where <configfile> is a shell configuration file:");
      configMessage();
    }
    
    String line;
    System.out.print("Minerva Shell> ");
    while((line=in.readLine())!=null) {
      if (!line.equalsIgnoreCase("QUIT"))
	doCommand(new StringTokenizer(line));
      else {
	if (something_to_save) {
	  System.out.print("Save current configuration? ");
	  line=in.readLine();
	  if (line.equalsIgnoreCase("yes"))
	    doCommand(new StringTokenizer("SAVE"));
	}
	break;
      }
      System.out.print("Minerva Shell> ");
    }
    
    return;
  }
  
  private static void doCommand(StringTokenizer command) throws IOException {
    String cmd;
    if (command.hasMoreTokens()) cmd = command.nextToken();
    else cmd = "HELP";
    if (cmd.equalsIgnoreCase("SELECT")) {
      if (command.hasMoreTokens()) {
	something_to_save = true;
	NFfile = new File(command.nextToken());
	System.out.println("NF File Selected.");
	compiled=false;
      }
    }
    else if (cmd.equalsIgnoreCase("ROOT")) {
      if (command.hasMoreTokens()) {
	something_to_save = true;
	root = command.nextToken();
      }
    }
    else if (cmd.equalsIgnoreCase("SHOW")) {
      saveConfigFile(System.out);
    }
    else if (cmd.equalsIgnoreCase("SAVE")) {
      if (NFfile!=null) {
	String filename;
	int indexOfLastPoint = NFfile.toString().lastIndexOf('.');
	if (indexOfLastPoint!=-1) {
	  filename = NFfile.toString().substring(0,indexOfLastPoint)+".cfg";
	}
	else filename=NFfile.toString()+".cfg";
	System.out.println("Name of config file ["+filename+"]: ");
	String typed=in.readLine();
	if (typed!=null && typed.length()!=0) {
	  filename=typed;
	}
	try {
	  PrintStream out = new PrintStream(new FileOutputStream(filename));
	  saveConfigFile(out);
	  out.close();
	}
	catch (IOException ioe) {
	  System.out.println("Cannot save config file: "+ioe.getMessage());
	}
      }
      else {
	System.out.println("None NF file specified");
      }

    }
    else if (cmd.equalsIgnoreCase("ADDSOURCES")) {
      while (command.hasMoreTokens()) {
	something_to_save = true;
	sources.add(command.nextToken());
      }
    }
    else if (cmd.equalsIgnoreCase("SHOWSOURCES")) {
      showSources();
    }
    else if (cmd.equalsIgnoreCase("DELETESOURCES")) {
      List toRemove = new LinkedList();
      String number;
      while (command.hasMoreTokens()) {
	something_to_save = true;
	number = command.nextToken();
	try {
	  int n = Integer.parseInt(number);
	  if (n<0 || n>=sources.size()) {
	    System.out.println(n+": Unknown number of source.");
	  }
	  else toRemove.add(sources.get(n));
	}
	catch (NumberFormatException e) {
	  System.out.println("Skipping "+number+": Bad input.");
	  continue;
	}
      }
      for(Iterator j=toRemove.iterator(); j.hasNext(); ) {
	sources.remove(j.next());
      }
      showSources();
    }
    else if (cmd.equalsIgnoreCase("GENERATE")) {
      if (NFfile==null) {
	System.out.println("None wrapper specification has been selected");
      }
      else {
	if (command.hasMoreTokens()) {
	  something_to_save = true;
	  destFile = new File(command.nextToken());
	}
	else {
	  if (destFile==null) {
	    something_to_save = true;
	    int lastPoint = NFfile.toString().lastIndexOf('.');
	    destFile = new File(NFfile.toString().substring(0,lastPoint)+".java");
	    System.out.println("Wrapper generated in: "+destFile.toString());
	  }
	}
	String[] args = new String[2];
	args[0] = NFfile.toString(); args[1] = destFile.toString();
	try {
	  mwg.generate(compiler,args);
	  compiled=true;
	}
	catch (Exception ce) {
	  System.out.println("Wrapper generation failed:");
	  System.out.println(ce.getMessage());
	  compiled=false;
	}
      }
    } 
    else if (cmd.equalsIgnoreCase("WRAP")) {
      if (root!=null) {
	if (compiled) {
	  String[] args = new String[sources.size()+1];
	  args[0] = root;
	  int k=1;
	  for(Iterator i=sources.iterator(); i.hasNext();) {
	    args[k++] = (String)i.next();
	  }
	  String classFile = destFile.getName();
	  String className = classFile.substring(0,classFile.lastIndexOf('.'));
	  System.out.println("Class Name: "+className);/**/
	  Class[] mainPars = new Class[1];
	  mainPars[0] = args.getClass();
	  try {
	    Class wrapperClass = Class.forName(className);
	    Method mainMethod = wrapperClass.getMethod("main",mainPars);
	    Object[] mainArgs = new Object[1];
	    mainArgs[0] = args;
	    mainMethod.invoke(null,mainArgs);
	  }
	  catch (ClassNotFoundException cnf) {
	    System.out.println("Cannot load wrapper class:");
	    System.out.println(cnf.getMessage());
	    System.out.println("Change CLASSPATH enviroment variable so to include class just generated");
	  }
	  catch (NoSuchMethodException nsme) {
	    System.out.println("Chosen root production does not exist"+nsme.getMessage());
	  }
	  catch (Exception e) {
	    System.out.println(e.getMessage());
	  }
	}
	else System.out.println("Wrapped not yet generated");
      }
      else System.out.println("You must specify a root production to apply");
    }
    else if (cmd.equalsIgnoreCase("ABOUT")) {
      welcomeMessage();
    }
    else if (cmd.equalsIgnoreCase("HELP")) {    
      System.out.println();
      System.out.println("Araneus MinervaShell Commands:");
      System.out.println(
      "-----------------------------------------------------------------------------");
      System.out.println(
      "SELECT <NFfile>            : select a NF file (wrapper specification)");
      System.out.println(
      "ROOT   <StartingSymbol>    : select root production to apply to wrap sources");
      System.out.println(
      "SHOW                       : show current shell configuration");
      System.out.println(
      "GENERATE [<DEST>]          : generate and compile java wrapper"); 
      System.out.println(
      "WRAP                       : run wrapping of <SOURCES>");
      System.out.println(
      "ADDSOURCES [<SOURCE>...]   : add local files or URL to list of sources");
      System.out.println(
      "SHOWSOURCES                : show list of sources");
      System.out.println(
      "DELETESOURCES [<S_NUMBER>] : delete sources from list");
      System.out.println(
      "HELP                       : show this help screen");
      System.out.println(
      "ABOUT                      : show copyright message and info");
      System.out.println(
      "SAVE                       : save shell configuration file");
      System.out.println(
      "QUIT                       : exit from this shell");
      System.out.println();
    }
    else {
      System.out.println("Unrecognized command; type 'help' for help.");
      System.out.println();
    }
    return;
  }
  
  private static void welcomeMessage() {
    System.out.println();
    System.out.println();
    System.out.println("     *************************************************************");
    System.out.println("     *                                                           *");
    System.out.println("     *  Copyright (c) 1997, 1998, 1999                           *");
    System.out.println("     *  Araneus Group and University 'Roma Tre', Rome, ITALY.    *");
    System.out.println("     *                                                           *");
    System.out.println("     *  All Rights Reserved.                                     *");
    System.out.println("     *                                                           *");
    System.out.println("     *  Permission to use, copy, and distribute this software    *");
    System.out.println("     *  and its documentation for NON-COMMERCIAL purposes and    *");
    System.out.println("     *  without fee is hereby granted provided  that this        *");
    System.out.println("     *  copyright notice appears in all copies.                  *");
    System.out.println("     *                                                           *");
    System.out.println("     *  (for more info see  http://www.dia.uniroma3.it/Araneus)  *");
    System.out.println("     *                                                           *");
    System.out.println("     *************************************************************");
    System.out.println();
  }

  private static void configMessage() {
    System.out.println(COMPILER+" <command line to compile java sources>");
    System.out.println(NFFILE  +"       <wrapper specification.NF>");
    System.out.println(ROOT    +"     <root production to apply>");
    System.out.println(DEST    +"     <wrapper generated.java>");
    System.out.println(SOURCES);
    System.out.println("source_0");
    System.out.println("source_1");
    System.out.println(" ... ");
    System.out.println("source_n");
    System.out.println();
    System.out.println(" where each source may be a local file or an URL");
    System.out.println();
  }

  private static void parseConfigFile(File config) {
    LineNumberReader reader = null;
    try {
      reader = new LineNumberReader(new FileReader(config));
      StringTokenizer parser;
      String line;
      
      File newNFfile=null, newDestFile=null;
      String newRoot=null;
      String[] newCompiler=null;
      List newSources = new LinkedList();
      
      while ((line=reader.readLine())!=null) {
	parser = new StringTokenizer(line);
	String firstToken = parser.nextToken();
	if (firstToken.equalsIgnoreCase(COMPILER)) {
	  newCompiler = new String[parser.countTokens()];
	  int k=0;
	  while (parser.hasMoreTokens())
	    newCompiler[k++] = parser.nextToken();
	}
	else if (firstToken.equalsIgnoreCase(NFFILE)) {
	  newNFfile = new File(parser.nextToken());
	}
	else if (firstToken.equalsIgnoreCase(ROOT)) {
	  newRoot = parser.nextToken();
	}
	else if (firstToken.equalsIgnoreCase(DEST)) {
	  newDestFile = new File(parser.nextToken());
	}
	else if (firstToken.equalsIgnoreCase(SOURCES)) {
	  while ((line=reader.readLine())!=null) {
	    newSources.add(line);
	  }
	}
	else System.out.println("Error parsing config file at line "+
				reader.getLineNumber());
      }
      if (newNFfile==null) {
	System.out.println("You must specify an NF field in a config file!");
      }
      else {
	NFfile = newNFfile;
	compiler = newCompiler;
	root = newRoot;
	destFile = newDestFile;
	sources = newSources;
      }
    }
    catch (NoSuchElementException nsee) {
      System.out.println("Error parsing config file at line "+
			 reader.getLineNumber());      
    }
    catch (FileNotFoundException fnfe) {
      System.out.println("Cannot find config file");
    }
    catch (IOException ioe) {
      System.out.println("Error reading config file: "+ioe.getMessage());
    }
  }

  private static void saveConfigFile(PrintStream out) {
    if (compiler!=null) {
      out.print(COMPILER+" ");
      if (compiler.length>0) out.print(compiler[0]);
      for(int i=1; i<compiler.length; i++) {
	out.print(" "+compiler[i]);
      }
      out.println();
    }
    if (NFfile!=null) out.println(NFFILE+"       "+NFfile);
    if (root!=null) out.println(ROOT+"     "+root);
    if (destFile!=null) out.println(DEST+"     "+destFile);
    out.println(SOURCES);
    for(Iterator i=sources.iterator(); i.hasNext(); ) {
      out.println((String)i.next());
    }
  }

  
  private static void showSources() {
    for(ListIterator i=sources.listIterator(); i.hasNext();) {
      System.out.println(i.nextIndex()+")\t"+i.next());
    }
  }

}
